︠7cf3f211-e1c5-4591-8184-4609451b1461︠
def add_affine(P,Q,A):
    """addition algorithm for affine points an elliptic curve in short Weierstrass form -- handles all cases (including point at infinity)"""
    if P == O: return Q
    if Q == O: return P
    x1=P[0]; y1=P[1]; x2=Q[0]; y2=Q[1];
    if x1 != x2:
        m = (y2-y1)/(x2-x1)         # usual case: P and Q are distinct and not opposites
    else:
        if y1 == -y2: return O      # P = -Q (includes case where P=Q is 2-torsion)
        m = (3*x1^2+A) / (2*y1)     # P = Q so we are doubling
    x3 = m^2-x1-x2
    y3 = m*(x1-x3)-y1
    return (x3,y3,1)

def negate(P):
    """negates the point P on an elliptic curve in short Weierstrass form"""
    if P == O: return O
    return (P[0],-P[1],P[2])
    
def dbl_affine(P,A):
    """doubles an affine non-2-torsion point on an elliptic curve in short Weierstrass form"""
    x1 = P[0];  y1 = P[1]
    m = (3*x1^2+A) / (2*y1)
    x3 = m^2-2*x1
    y3 = m*(x1-x3)-y1
    return (x3,y3,1)
    
def random_point(A,B):
    """generates a random affine point on the elliptic curve y^2 = x^2 + Ax + B"""
    F=A.parent()
    x0=F.random_element(); t=(x0^3+A*x0+B)
    while not t.is_square():
        x0=F.random_element(); t=(x0^3+A*x0+B)
    return (x0,sqrt(t),1)
︡684a600d-789a-4f49-8d27-b6fbaa2f2ffc︡
︠ab16c6c7-9f86-4e74-826f-16843f80c7a2︠
p=random_prime(2^512,lbound=2^511)
F=GF(p)
A=F(3); B=F.random_element()
P0=random_point(A,B)
Q0=random_point(A,B)
print "starting tests"
timeit('add_affine(P0,Q0,A)',number=50000)
timeit('add_affine(P0,P0,A)',number=50000)
timeit('dbl_affine(P0,A)',number=50000) 
︡3ba3c817-c726-44eb-95de-ad3c9dd7ac57︡{"stdout":"starting tests\n"}︡{"stdout":"50000 loops, best of 3: 13.4 µs per loop"}︡{"stdout":"\n"}︡{"stdout":"50000 loops, best of 3: 16.4 µs per loop"}︡{"stdout":"\n"}︡{"stdout":"50000 loops, best of 3: 15.7 µs per loop"}︡{"stdout":"\n"}︡
︠8215e6a0-310e-4edd-989b-0e14ae66c8d2︠
def add_edwards(P,Q,d):
    """adds two projective points on an elliptic curve in Edwards form (assumes d non-square, so operation is complete)"""
    # uses unoptimized formula derived in class (could save 1M with optimization)
    x1=P[0]; y1=P[1]; z1=P[2]; x2=Q[0]; y2=Q[1]; z2=Q[2]
    x1y2 = x1*y2; x2y1 = x2*y1
    r = z1*z2; rr = r^2; s = x1y2+x2y1; t = d*x1y2*x2y1; u = y1*y2 - x1*x2
    v = rr+t; w = rr-t
    x3 = r*s*w
    y3 = r*u*v
    z3 = v*w
    return (x3,y3,z3)
    
def dbl_edwards(P):
    """doubles a projective point on an elliptic curve in Edwards form"""
    # uses optimized formula from Bernstein-Lange
    x1 = P[0]; y1 = P[1]; z1 = P[2]
    B=(x1+y1)^2; C=x1^2; D=y1^2; E = C+D;
    J = E-2*z1^2; x3 = (B-E)*J; y3 = E*(C-D); z3 = E*J
    return (x3,y3,z3)
    
def neg_edwards(P):
    return (-P[0],P[1],P[2])
    
def random_edwards_point(d):
    """generates a random projective point on the Edwards curve x^2 + y^2 = 1+d*x^2*y^2 (assumes d non-square)"""
    F=d.parent()
    x0=F.random_element(); t=(1-x0^2)/(1-d*x0^2)
    while not t.is_square():
        x0=F.random_element(); t=(1-x0^2)/(1-d*x0^2)
    return (x0,sqrt(t),1)
    
def on_edwards_curve (P,d):
    x0=P[0]; y0=P[1]; z0=P[2]
    return z0^2*(x0^2+y0^2) == z0^4 + d*x0^2*y0^2
    
def is_id_edwards (P):
    if P[0] == 0 and P[1] == P[2]: return true
    else: return false 
︡ad6f0ddf-d6dd-4e57-bb44-0f2b2f7d0da5︡
︠380a6df9-eb7b-47b7-b9f0-e257f145293c︠
p = 17
F=GF(p)
d = F.multiplicative_generator()
print "Points on Edwards curve x^2 + y^2 = 1 + %dx^2y^2 over F_%d"%(d,p)
for a in F:
    for b in F:
        if on_edwards_curve([a,b,1],d): print [a,b] 
︡fca75ebb-ef00-41c4-80d2-741f86719d6d︡{"stdout":"Points on Edwards curve x^2 + y^2 = 1 + 3x^2y^2 over F_17\n"}︡{"stdout":"[0, 1]\n[0, 16]\n[1, 0]\n[2, 5]\n[2, 12]\n[3, 4]\n[3, 13]\n[4, 3]\n[4, 14]\n[5, 2]\n[5, 15]\n[7, 7]\n[7, 10]\n[10, 7]\n[10, 10]\n[12, 2]\n[12, 15]\n[13, 3]\n[13, 14]\n[14, 4]\n[14, 13]\n[15, 5]\n[15, 12]\n[16, 0]\n"}︡
︠fd85897a-9840-4cb2-8256-11d7979d59f0︠
p=random_prime(2^512,lbound=2^511)
F=GF(p)
d=F(-1);
while d.is_square(): d-=1;
P0=random_edwards_point(d)
Q0=random_edwards_point(d)
print "starting tests"
timeit('add_edwards(P0,Q0,d)',number=50000)
timeit('add_edwards(P0,P0,d)',number=50000)
timeit('dbl_edwards(P0)',number=50000)
︡2a61fc97-9301-45da-946e-5b4b588f9ca6︡{"stdout":"starting tests\n"}︡{"stdout":"50000 loops, best of 3: 13.1 µs per loop"}︡{"stdout":"\n"}︡{"stdout":"50000 loops, best of 3: 14.4 µs per loop"}︡{"stdout":"\n"}︡{"stdout":"50000 loops, best of 3: 10.3 µs per loop"}︡{"stdout":"\n"}︡
︠3b6c1ec8-69d9-40a0-84ad-0c85ebeba396︠









